@@ -3,7 +3,7 @@ BEGIN {
   $Dist::Zilla::Plugin::CoderwallEndorse::AUTHORITY = 'cpan:YANICK';
 }
 {
-  $Dist::Zilla::Plugin::CoderwallEndorse::VERSION = '0.1.2';
+  $Dist::Zilla::Plugin::CoderwallEndorse::VERSION = '0.2.0';
 }
 # ABSTRACT: Adds a Coderwall 'endorse' button to README Markdown file
 
@@ -15,9 +15,11 @@ use Moose;
 
 use List::Util qw/ first /;
 
+use Dist::Zilla::Role::File::ChangeNotification;
+
 with qw/
     Dist::Zilla::Role::Plugin
-    Dist::Zilla::Role::InstallTool
+    Dist::Zilla::Role::FileMunger
 /;
 
 has users => (
@@ -25,6 +27,12 @@ has users => (
     isa => 'Str',
 );
 
+has "filename" => (
+    isa => 'Str',
+    is => 'ro',
+    default => 'README.mkdn',
+);
+
 has mapping => (
     traits => [ 'Hash' ],
     is => 'ro',
@@ -46,18 +54,62 @@ has mapping => (
     },
 );
 
-sub setup_installer {
+has "_last_content" => (
+    isa => 'Str',
+    is => 'rw',
+    default => '',
+);
+
+sub munge_files {
     my $self = shift;
 
-    my $readme = first { $_->name eq 'README.mkdn' } 
-                           @{ $self->zilla->files } or return;
+    my $filename = $self->filename;
+    my( $file ) = grep { $_->name eq $filename } @{ $self->zilla->files }
+        or return $self->log([ "file '%s' not found", $filename ]);
+
+    $self->munge_file($file);
+    $self->watch($file);
+}
+
+sub watch {
+    my( $self, $file ) = @_;
+    
+    Dist::Zilla::Role::File::ChangeNotification->meta->apply($file)
+        unless $file->does('Dist::Zilla::Role::File::ChangeNotification');
+
+    my $plugin = $self;
+    $file->on_changed(sub {
+        my ($self, $newcontent) = @_;
+
+        $self->_content_checksum(0);
+        $self->watch_file;
+
+        # If the new content is actually different, recalculate
+        # the content based on the updates.
+        return if $newcontent eq $plugin->_last_content;
+
+        $plugin->log_debug('someone tried to munge ' . $self->name . ' after we read from it. Making modifications again...');
+        $plugin->munge_file($file);
+    });
+
+    $file->watch_file;
+}
+
+sub munge_file {
+    my( $self, $file ) = @_;
+
+    $self->log_debug([ 'CoderwallEndorse updating contents of %s in dist', $file->name ]);
 
     my $new_content;
 
-    for my $line ( split /\n/, $readme->content ) {
+    for my $line ( split /\n/, $file->content ) {
         if ( $line=~ /^# AUTHOR/ ... $line =~ /^#/ ) {
             for my $auth ( $self->authors ) {
-                next if -1 == index $line, $auth;
+
+                # author not mentioned, or endorse link already there
+                next if -1 == index $line, $auth
+                     or -1 != index $line, '[endorse]';
+
                 $line .= sprintf " [![endorse](http://api.coderwall.com/%s/endorsecount.png)](http://coderwall.com/%s)",
                                 ( $self->cd_user($auth) ) x 2;
 
@@ -66,7 +118,8 @@ sub setup_installer {
         $new_content .= $line."\n";
     }
 
-    $readme->content($new_content);
+    $self->_last_content($new_content);
+    $file->content($new_content);
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -78,13 +131,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dist::Zilla::Plugin::CoderwallEndorse - Adds a Coderwall 'endorse' button to README Markdown file
 
 =head1 VERSION
 
-version 0.1.2
+version 0.2.0
 
 =head1 SYNOPSIS
 
@@ -94,6 +149,7 @@ version 0.1.2
     [ReadmeMarkdownFromPod]
 
     [CoderwallEndorse]
+    filename = README.mkdn
     users = coderwall_name : author name, other_cw_name : other author
 
 =head1 DESCRIPTION
@@ -104,7 +160,7 @@ given.
 
 =head1 SEE ALSO
 
-L<www.coderwall.com>
+L<http://www.coderwall.com>
 
 L<Dist::Zilla::Plugin::ReadmeMarkdownFromPod>
 
@@ -115,7 +171,7 @@ Original blog entry: L<http://babyl.dyndns.org/techblog/entry/coderwall-button>
 
 =head1 AUTHOR
 
-Yanick Champoux <yanick@babyl.dyndns.org>
+Yanick Champoux <yanick@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
